@@ -2,8 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001, 2002 Sun Microsystems Inc.,
- * Copyright 2001, 2002 Ximian, Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.,
+ * Copyright 2001, 2002, 2003 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -49,23 +50,12 @@ static gboolean registry_died = FALSE;
 static gboolean atk_listeners_registered = FALSE;
 static guint toplevel_handler;
 
-/* NOT YET USED
-   static GQuark atk_quark_property_changed_name;
-   static GQuark atk_quark_property_changed_description;
-   static GQuark atk_quark_property_changed_parent;
-   static GQuark atk_quark_property_changed_role;
-   static GQuark atk_quark_property_changed_table_caption;
-   static GQuark atk_quark_property_changed_table_column_description;
-   static GQuark atk_quark_property_changed_table_row_description;
-   static guint atk_signal_property_changed;
-*/
-
 static guint atk_signal_text_changed;
-static guint atk_signal_child_changed;
+static guint atk_signal_children_changed;
 static guint atk_signal_active_descendant_changed;
+static guint atk_signal_text_selection_changed;
 
 /* NOT YET USED
-   static guint atk_signal_text_selection_changed;
    static guint atk_signal_row_reordered;
    static guint atk_signal_row_inserted;
    static guint atk_signal_row_deleted;
@@ -74,6 +64,8 @@ static guint atk_signal_active_descendant_changed;
    static guint atk_signal_column_deleted;
 */
 
+static guint atk_signal_link_selected;
+
 static Accessibility_Registry spi_atk_bridge_get_registry (void);
 static void     spi_atk_bridge_do_registration         (void);
 static void     spi_atk_bridge_toplevel_added          (AtkObject             *object,
@@ -124,13 +116,17 @@ extern void gnome_accessibility_module_shutdown (void);
 static void
 spi_atk_bridge_init_event_type_consts ()
 {
-  atk_signal_child_changed = g_signal_lookup ("child_changed", 
+  atk_signal_children_changed = g_signal_lookup ("children_changed", 
 					      ATK_TYPE_OBJECT);
   atk_signal_text_changed = g_signal_lookup ("text_changed", 
 					     ATK_TYPE_TEXT);
   atk_signal_active_descendant_changed = 
          g_signal_lookup ("active_descendant_changed", 
 		          ATK_TYPE_OBJECT);
+  atk_signal_link_selected = g_signal_lookup ("link_selected", 
+					      ATK_TYPE_HYPERTEXT);
+  atk_signal_text_selection_changed = g_signal_lookup ("text_selection_changed", 
+					      ATK_TYPE_TEXT);
 }
 
 static int
@@ -360,6 +356,7 @@ spi_atk_register_event_listeners (void)
   add_signal_listener ("Gtk:AtkTable:column-reordered");
   add_signal_listener ("Gtk:AtkTable:column-deleted");
   add_signal_listener ("Gtk:AtkTable:model-changed");
+  add_signal_listener ("Gtk:AtkHypertext:link-selected");
 /*
  * May add the following listeners to implement preemptive key listening for GTK+
  *
@@ -561,6 +558,13 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
 {
   AtkPropertyValues *values;
   GObject *gobject;
+  const gchar *prop_name;
+  CORBA_any any;
+  const gchar *sp = NULL;
+  AtkObject *ao;
+  SpiAccessible *s_ao = NULL;
+  CORBA_Object c_obj;
+  gint i;
 
 #ifdef SPI_BRIDGE_DEBUG
   GSignalQuery signal_query;
@@ -581,8 +585,101 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
   gobject = g_value_get_object (param_values + 0);
   values = (AtkPropertyValues*) g_value_get_pointer (param_values + 1);
 
-  spi_atk_emit_eventv (gobject, 0, 0, NULL,
-		       "object:property-change:%s", values->property_name);
+  prop_name = values->property_name;
+  if (strcmp (prop_name, "accessible-name") == 0)
+    {
+      sp = atk_object_get_name (ATK_OBJECT (gobject));
+      spi_init_any_string (&any, (gchar **)&sp);
+    }
+  else if (strcmp (prop_name, "accessible-description") == 0)
+    {
+      sp = atk_object_get_description (ATK_OBJECT (gobject));
+      spi_init_any_string (&any, (gchar **)&sp);
+    }
+  else if (strcmp (prop_name, "accessible-parent") == 0)
+    {
+      ao = atk_object_get_parent (ATK_OBJECT (gobject));
+      if (ao) 
+        {
+          s_ao = spi_accessible_new (ao);
+          c_obj = BONOBO_OBJREF (s_ao);
+          spi_init_any_object (&any, &c_obj);
+	}
+      else
+        {
+          spi_init_any_nil (&any);
+        }
+    }
+  else if (strcmp (prop_name, "accessible-table-summary") == 0)
+    {
+      ao = atk_table_get_summary (ATK_TABLE (gobject));
+      if (ao) 
+        {
+          s_ao = spi_accessible_new (ao);
+          c_obj = BONOBO_OBJREF (s_ao);
+          spi_init_any_object (&any, &c_obj);
+	}
+      else
+        {
+          spi_init_any_nil (&any);
+        }
+    }
+  else if (strcmp (prop_name, "accessible-table-column-header") == 0)
+    {
+      i = g_value_get_int (&(values->new_value));
+      ao = atk_table_get_column_header (ATK_TABLE (gobject), i);
+      if (ao) 
+        {
+          s_ao = spi_accessible_new (ao);
+          c_obj = BONOBO_OBJREF (s_ao);
+          spi_init_any_object (&any, &c_obj);
+	}
+      else
+        {
+          spi_init_any_nil (&any);
+        }
+    }
+  else if (strcmp (prop_name, "accessible-table-row-header") == 0)
+    {
+      i = g_value_get_int (&(values->new_value));
+      ao = atk_table_get_row_header (ATK_TABLE (gobject), i);
+      if (ao) 
+        {
+          s_ao = spi_accessible_new (ao);
+          c_obj = BONOBO_OBJREF (s_ao);
+          spi_init_any_object (&any, &c_obj);
+	}
+      else
+        {
+          spi_init_any_nil (&any);
+        }
+    }
+  else if (strcmp (prop_name, "accessible-table-row-description") == 0)
+    {
+      i = g_value_get_int (&(values->new_value));
+      sp = atk_table_get_row_description (ATK_TABLE (gobject), i);
+      spi_init_any_string (&any, (gchar **)&sp);
+    }
+  else if (strcmp (prop_name, "accessible-table-column-description") == 0)
+    {
+      i = g_value_get_int (&(values->new_value));
+      sp = atk_table_get_column_description (ATK_TABLE (gobject), i);
+      spi_init_any_string (&any, (gchar **)&sp);
+    }
+  else if (strcmp (prop_name, "accessible-table-caption-object") == 0)
+    {
+      ao = atk_table_get_caption (ATK_TABLE (gobject));
+      sp = atk_object_get_name (ao);
+      spi_init_any_string (&any, (gchar **)&sp);
+    }
+  else
+    {
+      spi_init_any_nil (&any);
+    }
+
+  spi_atk_emit_eventv (gobject, 0, 0, &any,
+		       "object:property-change:%s", prop_name);
+
 
   return TRUE;
 }
@@ -644,8 +741,14 @@ spi_init_keystroke_from_atk_key_event (Accessibility_DeviceEvent  *keystroke,
   keystroke->modifiers = (CORBA_unsigned_short) (event->state & 0xFFFF);
   if (event->string)
     {
+      gunichar c;
+
       keystroke->event_string = CORBA_string_dup (event->string);
-      keystroke->is_text = CORBA_TRUE;
+      c = g_utf8_get_char_validated (event->string, -1);
+      if (c > 0 && g_unichar_isprint (c))
+        keystroke->is_text = CORBA_TRUE;
+      else
+        keystroke->is_text = CORBA_FALSE;
     }
   else
     {
@@ -704,6 +807,7 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   CORBA_Object c_obj;
   char *sp = NULL;
   AtkObject *ao;
+  AtkText *text;
   gint detail1 = 0, detail2 = 0;
   SpiAccessible *s_ao = NULL;
 #ifdef SPI_BRIDGE_DEBUG
@@ -741,12 +845,40 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
       c_obj = BONOBO_OBJREF (s_ao);
       spi_init_any_object (&any, &c_obj);
     }
-  else
+  else if (signal_query.signal_id == atk_signal_link_selected)
     {
       if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
         detail1 = g_value_get_int (param_values + 1);
-      if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
-        detail2 = g_value_get_int (param_values + 2);
+      spi_init_any_nil (&any);
+    }
+  else if ((signal_query.signal_id == atk_signal_children_changed) && gobject)
+    {
+      ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
+			        	    detail1);
+      if (ao) 
+        {
+          s_ao = spi_accessible_new (ao);
+          c_obj = BONOBO_OBJREF (s_ao);
+          spi_init_any_object (&any, &c_obj);
+	  g_object_unref (ao);
+	}
+      else
+	{
+	  spi_init_any_nil (&any);
+	}
+    }
+  else
+    {
+      if (n_param_values >= 2)
+        {
+          if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
+            detail1 = g_value_get_int (param_values + 1);
+          if (n_param_values >= 3)
+            {
+              if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
+                detail2 = g_value_get_int (param_values + 2);
+            }
+        }
 
       if (signal_query.signal_id == atk_signal_text_changed)
         {
@@ -755,21 +887,12 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
 			          detail1+detail2);
           spi_init_any_string (&any, &sp);
         }
-      else if ((signal_query.signal_id == atk_signal_child_changed) && gobject)
+      else if (signal_query.signal_id == atk_signal_text_selection_changed)
         {
-          ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
-					        detail1);
-          if (ao) 
-	    {
-              s_ao = spi_accessible_new (ao);
-              c_obj = BONOBO_OBJREF (s_ao);
-              spi_init_any_object (&any, &c_obj);
-	      g_object_unref (ao);
-	    }
-          else
-	    {
-	      spi_init_any_nil (&any);
-	    }
+          text = ATK_TEXT (gobject);
+ 
+          /* Return NULL as the selected string */
+	  spi_init_any_nil (&any);
         }
       else
         {
