@@ -1,3 +1,26 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 /*
  *
  * AccessibleApplication function prototypes
@@ -76,7 +99,7 @@ AccessibleApplication_getVersion (AccessibleApplication *obj)
     Accessibility_Application__get_version (CSPI_OBJREF (obj),
 					    cspi_ev ());
 
-  cspi_return_val_if_ev ("getVersion", NULL);
+  cspi_return_val_if_ev ("version", NULL);
 
   return retval;
 }
@@ -102,7 +125,7 @@ AccessibleApplication_getID (AccessibleApplication *obj)
   retval = Accessibility_Application__get_id (CSPI_OBJREF (obj),
 					      cspi_ev ());
 
-  cspi_return_val_if_ev ("get_id", 0);
+  cspi_return_val_if_ev ("id", 0);
 
   return retval;
 }
